@@ -4,7 +4,7 @@
  * Eddie Kohler
  *
  * Copyright (c) 1999-2000 Massachusetts Institute of Technology
- * Copyright (c) 2001-2004 Eddie Kohler
+ * Copyright (c) 2001-2005 Eddie Kohler
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -234,12 +234,21 @@ do_number_flags(char *pos, char *after_last, int base, int flags,
 }
 
 String
-ErrorHandler::make_text(Seriousness, const char *s, va_list val)
+ErrorHandler::make_text(Seriousness seriousness, const char *s, va_list val)
 {
   StringAccum msg;
+
   char numbuf[NUMBUF_SIZE];	// for numerics
-  String placeholder;		// to ensure temporaries aren't destroyed
   numbuf[NUMBUF_SIZE-1] = 0;
+
+  String placeholder;		// to ensure temporaries aren't destroyed
+
+  // prepend 'warning: ' to every line if appropriate
+  String begin_lines;
+  if (seriousness >= ERR_MIN_WARNING && seriousness < ERR_MIN_ERROR) {
+    begin_lines = String::stable_string("warning: ", 9);
+    msg << begin_lines;
+  }
   
   // declare and initialize these here to make gcc shut up about possible 
   // use before initialization
@@ -251,12 +260,23 @@ ErrorHandler::make_text(Seriousness, const char *s, va_list val)
   while (1) {
     
     const char *pct = strchr(s, '%');
+
+    // handle begin_lines
+    const char *nl;
+    while (begin_lines && (nl = strchr(s, '\n')) && (!pct || nl < pct)) {
+      msg.append(s, nl + 1 - s);
+      if (nl[1])
+	msg.append(begin_lines.data(), begin_lines.length());
+      s = nl + 1;
+    }
+    
     if (!pct) {
-      if (*s) msg << s;
+      if (*s)
+	msg << s;
       break;
     }
     if (pct != s) {
-      memcpy(msg.extend(pct - s), s, pct - s);
+      msg.append(s, pct - s);
       s = pct;
     }
     
@@ -491,16 +511,10 @@ ErrorHandler::make_text(Seriousness, const char *s, va_list val)
 }
 
 String
-ErrorHandler::decorate_text(Seriousness seriousness, const String &landmark, const String &text)
+ErrorHandler::decorate_text(Seriousness, const String &landmark, const String &text)
 {
-  String new_text;
+  String new_text = text;
   
-  // prepend 'warning: ' to every line if appropriate
-  if (seriousness >= ERR_MIN_WARNING && seriousness < ERR_MIN_ERROR)
-    new_text = prepend_lines("warning: ", text);
-  else
-    new_text = text;
-
   // handle landmark
   if (landmark
       && !(landmark.length() > 2 && landmark[0] == '\\'
@@ -509,7 +523,7 @@ ErrorHandler::decorate_text(Seriousness seriousness, const String &landmark, con
     // fix landmark: skip trailing spaces and trailing colon
     int i, len = landmark.length();
     for (i = len - 1; i >= 0; i--)
-      if (!isspace(landmark[i]))
+      if (!isspace((unsigned char) landmark[i]))
 	break;
     if (i >= 0 && landmark[i] == ':')
       i--;
@@ -556,13 +570,13 @@ ErrorHandler::prepend_lines(const String &prepend, const String &text)
   StringAccum sa;
   const char *begin = text.begin();
   const char *end = text.end();
-  const char *nl;
-  while ((nl = std::find(begin, end, '\n')) < end) {
-    sa << prepend << text.substring(begin, nl + 1);
-    begin = nl + 1;
+  while (begin < end) {
+    const char *nl = std::find(begin, end, '\n');
+    if (nl < end)
+      nl++;
+    sa << prepend << text.substring(begin, nl);
+    begin = nl;
   }
-  if (begin < end)
-    sa << prepend << text.substring(begin, end);
   
   return sa.take_string();
 }
