@@ -1,6 +1,6 @@
 /* automatic.{cc,hh} -- code for automatic mode and interfacing with kpathsea
  *
- * Copyright (c) 2003-2004 Eddie Kohler
+ * Copyright (c) 2003-2005 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -88,7 +88,7 @@ kpsei_string(char* x)
 }
 
 static void
-look_for_writable_texdir(const char *path_variable)
+look_for_writable_texdir(const char *path_variable, bool create)
 {
     String path = kpsei_string(kpsei_path_expand(path_variable));
     while (path && !writable_texdir) {
@@ -97,6 +97,9 @@ look_for_writable_texdir(const char *path_variable)
 	path = path.substring(colon + 1, path.end());
 	if (access(texdir.c_str(), W_OK) >= 0)
 	    writable_texdir = texdir;
+	else if (create && errno != EACCES && mkdir(texdir.c_str(), 0777) >= 0)
+	    // create file if it doesn't exist already
+	    writable_texdir = texdir;
     }   
     if (writable_texdir && writable_texdir.back() != '/')
 	writable_texdir += "/";
@@ -105,11 +108,13 @@ look_for_writable_texdir(const char *path_variable)
 static void
 find_writable_texdir(ErrorHandler *errh, const char *)
 {
-    look_for_writable_texdir("$VARTEXMF");
+    look_for_writable_texdir("$TEXMFVAR", true);
     if (!writable_texdir)
-	look_for_writable_texdir("$TEXMF");
+	look_for_writable_texdir("$VARTEXMF", false);
+    if (!writable_texdir)
+	look_for_writable_texdir("$TEXMF", false);
     if (!writable_texdir) {
-	errh->warning("no writable directory found in $VARTEXMF or $TEXMF");
+	errh->warning("no writable directory found in $TEXMFVAR or $TEXMF");
 	errh->message("(You probably need to set your TEXMF environment variable; see\n\
 the manual for more information. The current TEXMF path is\n\
 '%s'.)", kpsei_string(kpsei_path_expand("$TEXMF")).c_str());
@@ -300,7 +305,7 @@ update_odir(int o, String file, ErrorHandler *errh)
 
 	// run script
 	if (mktexupd) {
-	    String command = mktexupd + " '" + writable_texdir + directory + "' '" + file + "'";
+	    String command = mktexupd + " " + shell_quote(writable_texdir + directory) + " " + shell_quote(file);
 	    int retval = system(command.c_str());
 	    if (retval == 127)
 		errh->error("could not run '%s'", command.c_str());
@@ -334,13 +339,13 @@ installed_type1(const String &otf_filename, const String &ps_fontname, bool allo
     if (!(force && allow_generate && otf_filename && otf_filename != "-" && getodir(O_TYPE1, errh))) {
 # endif
 	// look for .pfb and .pfa
-	String attempt = ps_fontname + ".pfb";
-	if (String found = kpsei_string(kpsei_find_file(attempt.c_str(), KPSEI_FMT_TYPE1)))
-	    return found;
-
-	attempt = ps_fontname + ".pfa";
-	if (String found = kpsei_string(kpsei_find_file(attempt.c_str(), KPSEI_FMT_TYPE1)))
-	    return found;
+	String file, path;
+	if ((file = ps_fontname + ".pfb", path = kpsei_string(kpsei_find_file(file.c_str(), KPSEI_FMT_TYPE1)))
+	    || (file = ps_fontname + ".pfa", path = kpsei_string(kpsei_find_file(file.c_str(), KPSEI_FMT_TYPE1)))) {
+	    if (verbose)
+		errh->message("Type 1 file %s found with kpathsea at %s", file.c_str(), path.c_str());
+	    return path;
+	}
 # if HAVE_AUTO_CFFTOT1
     }
 # endif
@@ -348,12 +353,11 @@ installed_type1(const String &otf_filename, const String &ps_fontname, bool allo
 
 #if HAVE_AUTO_CFFTOT1
     // if not found, and can generate on the fly, run cfftot1
-    if (allow_generate && otf_filename && otf_filename != "-"
-	&& getodir(O_TYPE1, errh)) {
+    if (allow_generate && otf_filename && otf_filename != "-" && getodir(O_TYPE1, errh)) {
 	String pfb_filename = odir[O_TYPE1] + "/" + ps_fontname + ".pfb";
 	if (pfb_filename.find_left('\'') >= 0 || otf_filename.find_left('\'') >= 0)
 	    return String();
-	String command = "cfftot1 '" + otf_filename + "' -n '" + ps_fontname + "' '" + pfb_filename + "'";
+	String command = "cfftot1 " + shell_quote(otf_filename) + " -n " + shell_quote(ps_fontname) + " " + shell_quote(pfb_filename);
 	int retval = mysystem(command.c_str(), errh);
 	if (retval == 127)
 	    errh->error("could not run '%s'", command.c_str());
@@ -386,13 +390,13 @@ installed_type1_dotlessj(const String &otf_filename, const String &ps_fontname,
     if (!(force && allow_generate && getodir(O_TYPE1, errh))) {
 # endif
 	// look for existing .pfb or .pfa
-	String attempt = j_ps_fontname + ".pfb";
-	if (String found = kpsei_string(kpsei_find_file(attempt.c_str(), KPSEI_FMT_TYPE1)))
-	    return found;
-
-	attempt = j_ps_fontname + ".pfa";
-	if (String found = kpsei_string(kpsei_find_file(attempt.c_str(), KPSEI_FMT_TYPE1)))
-	    return found;
+	String file, path;
+	if ((file = j_ps_fontname + ".pfb", path = kpsei_string(kpsei_find_file(file.c_str(), KPSEI_FMT_TYPE1)))
+	    || (file = j_ps_fontname + ".pfa", path = kpsei_string(kpsei_find_file(file.c_str(), KPSEI_FMT_TYPE1)))) {
+	    if (verbose)
+		errh->message("Type 1 file %s found with kpathsea at %s", file.c_str(), path.c_str());
+	    return path;
+	}
 # if HAVE_AUTO_T1DOTLESSJ
     }
 # endif
@@ -405,7 +409,7 @@ installed_type1_dotlessj(const String &otf_filename, const String &ps_fontname,
 	    String pfb_filename = odir[O_TYPE1] + "/" + j_ps_fontname + ".pfb";
 	    if (pfb_filename.find_left('\'') >= 0 || base_filename.find_left('\'') >= 0)
 		return String();
-	    String command = "t1dotlessj '" + base_filename + "' -n '" + j_ps_fontname + "' '" + pfb_filename + "'";
+	    String command = "t1dotlessj " + shell_quote(base_filename) + " -n " + shell_quote(j_ps_fontname) + " " + shell_quote(pfb_filename);
 	    int retval = mysystem(command.c_str(), errh);
 	    if (retval == 127)
 		errh->warning("could not run '%s'", command.c_str());
@@ -544,38 +548,74 @@ update_autofont_map(const String &fontname, String mapline, ErrorHandler *errh)
 
 #if HAVE_KPATHSEA && !WIN32
 	// run 'updmap' if present
-	String updmap_dir = getodir(O_MAP_PARENT, errh);
-	String updmap_file = updmap_dir + "/updmap";
-	if (automatic && access(updmap_file.c_str(), X_OK) >= 0) {
+	String updmap_dir, updmap_file;
+	if (automatic && (output_flags & G_UPDMAP))
+	    updmap_dir = getodir(O_MAP_PARENT, errh);
+	if (updmap_dir && (updmap_file = updmap_dir + "/updmap")
+	    && access(updmap_file.c_str(), X_OK) >= 0) {
 	    // want to run 'updmap' from its directory, can't use system()
 	    if (verbose)
 		errh->message("running %s", updmap_file.c_str());
-	    if (pid_t child = fork()) {
-# if HAVE_WAITPID
-		int status;
-		while (1) {
-		    pid_t answer = waitpid(child, &status, 0);
-		    if (answer >= 0)
-			break;
-		    else if (errno != EINTR)
-			errh->fatal("%s during wait", strerror(errno));
-		}
-		if (!WIFEXITED(status))
-		    errh->warning("%s exited abnormally", updmap_file.c_str());
-		else if (WEXITSTATUS(status) != 0)
-		    errh->warning("%s exited with status %d", updmap_file.c_str(), WEXITSTATUS(status));
-# else
-#  error "need waitpid() support: report this bug to the maintainer"
-# endif
-	    } else {
+	    
+	    pid_t child = fork();
+	    if (child < 0)
+		errh->fatal("%s during fork", strerror(errno));
+	    else if (child == 0) {
+		// change to updmap directory, run it
 		if (chdir(updmap_dir.c_str()) < 0)
 		    errh->fatal("%s: %s during chdir", updmap_dir.c_str(), strerror(errno));
 		if (execl("./updmap", updmap_file.c_str(), (const char*) 0) < 0)
 		    errh->fatal("%s: %s during exec", updmap_file.c_str(), strerror(errno));
 		exit(1);	// should never get here
 	    }
-	} else if (verbose)
+	    
+# if HAVE_WAITPID
+	    // wait for updmap to finish
+	    int status;
+	    while (1) {
+		pid_t answer = waitpid(child, &status, 0);
+		if (answer >= 0)
+		    break;
+		else if (errno != EINTR)
+		    errh->fatal("%s during wait", strerror(errno));
+	    }
+	    if (!WIFEXITED(status))
+		errh->warning("%s exited abnormally", updmap_file.c_str());
+	    else if (WEXITSTATUS(status) != 0)
+		errh->warning("%s exited with status %d", updmap_file.c_str(), WEXITSTATUS(status));
+# else
+#  error "need waitpid() support: report this bug to the maintainer"
+# endif
+	    goto ran_updmap;
+	}
+
+# if HAVE_AUTO_UPDMAP
+	// run system updmap
+	if (output_flags & G_UPDMAP) {
+	    String filename = map_file;
+	    int slash = filename.find_right('\'');
+	    if (slash >= 0)
+		filename = filename.substring(slash + 1);
+	    String command = "updmap --nomkmap --enable Map " + shell_quote(filename) + "; updmap";
+	    if (verbose)
+		command += " 1>&2";
+	    else
+		command += " >/dev/null 2>&1";
+	    int retval = mysystem(command.c_str(), errh);
+	    if (retval == 127)
+		errh->warning("could not run '%s'", command.c_str());
+	    else if (retval < 0)
+		errh->warning("could not run '%s': %s", command.c_str(), strerror(errno));
+	    else if (retval != 0)
+		errh->warning("'%s' exited with status %d;\nrun it manually to check for errors", command.c_str(), WEXITSTATUS(retval));
+	    goto ran_updmap;
+	}
+# endif
+	
+	if (verbose)
 	    errh->message("not running updmap");
+
+      ran_updmap: ;
 #endif
     }
 
