@@ -149,14 +149,27 @@ DvipsEncoding::encode(int e, PermString what)
 }
 
 int
-DvipsEncoding::encoding_of(PermString a) const
+DvipsEncoding::encoding_of(PermString what, bool encoding_required)
 {
+    int slot = -1;
     for (int i = 0; i < _e.size(); i++)
-	if (_e[i] == a)
-	    return i;
-    if (a == "||")
+	if (_e[i] == what) {
+	    slot = i;
+	    goto use_slot;
+	} else if (!_e[i] || _e[i] == dot_notdef)
+	    slot = i;
+    if (what == "||")
 	return _boundary_char;
-    return -1;
+    else if (!encoding_required || slot < 0)
+	return -1;
+  use_slot:
+    if (encoding_required) {
+	if (slot >= _encoding_required.size())
+	    _encoding_required.resize(slot + 1, false);
+	_encoding_required[slot] = true;
+	this->encode(slot, what);
+    }
+    return slot;
 }
 
 static String
@@ -291,26 +304,43 @@ find_ligkern_op(const String &s)
     return -1;
 }
 
+inline bool
+operator==(const DvipsEncoding::Ligature& l1, const DvipsEncoding::Ligature& l2)
+{
+    return l1.c1 == l2.c1 && l1.c2 == l2.c2;
+}
+
 int
-DvipsEncoding::parse_ligkern_words(Vector<String> &v, ErrorHandler *errh)
+DvipsEncoding::parse_ligkern_words(Vector<String> &v, int override, ErrorHandler *errh)
 {
     _file_had_ligkern = true;
     int op;
+    long l;
+    char *endptr;
     if (v.size() == 3) {
 	if (v[0] == "||" && v[1] == "=") {
 	    char *endptr;
-	    _boundary_char = strtol(v[2].c_str(), &endptr, 10);
+	    if (override > 0 || _boundary_char < 0)
+		_boundary_char = strtol(v[2].c_str(), &endptr, 10);
 	    if (*endptr == 0 && _boundary_char < _e.size())
 		return 0;
 	    else
 		return errh->error("parse error in boundary character assignment");
 	} else if (v[0] == "^^" && v[1] == "=") {
 	    char *endptr;
-	    _altselector_char = strtol(v[2].c_str(), &endptr, 10);
+	    if (override > 0 || _altselector_char < 0)
+		_altselector_char = strtol(v[2].c_str(), &endptr, 10);
 	    if (*endptr == 0 && _altselector_char < _e.size())
 		return 0;
 	    else
 		return errh->error("parse error in altselector character assignment");
+	} else if ((l = strtol(v[0].c_str(), &endptr, 0)) && endptr == v[0].end() && v[1] == "=") {
+	    if (l >= 0 && l < 256) {
+		if (override > 0 || !_e[l])
+		    encode(l, v[2]);
+		return 0;
+	    } else
+		return errh->error("encoding value '%d' out of range", l); 
 	} else if ((op = find_ligkern_op(v[1])) >= J_NOKERN) {
 	    int av = (v[0] == "*" ? J_ALL : encoding_of(v[0]));
 	    if (av < 0)
@@ -319,30 +349,38 @@ DvipsEncoding::parse_ligkern_words(Vector<String> &v, ErrorHandler *errh)
 	    if (bv < 0)
 		return errh->warning("'%s' has no encoding, ignoring %s", v[2].c_str(), nokern_names[op - J_NOKERN]);
 	    Ligature lig = { av, bv, op, 0 };
-	    _lig.push_back(lig);
+	    Ligature *what = std::find(_lig.begin(), _lig.end(), lig);
+	    if (override > 0 && what < _lig.end())
+		*what = lig;
+	    else if (what == _lig.end())
+		_lig.push_back(lig);
 	    return 0;
 	} else
 	    return -1;
     } else if (v.size() == 4 && (op = find_ligkern_op(v[2])) >= J_LIG
 	       && op <= J_CLIGC_SS) {
-	int av = encoding_of(v[0]);
+	int av = encoding_of(v[0], override > 0);
 	if (av < 0)
-	    return errh->warning("'%s' has no encoding, ignoring ligature", v[0].c_str());
-	int bv = encoding_of(v[1]);
+	    return (override > 0 ? errh->warning("'%s' has no encoding, ignoring ligature", v[0].c_str()) : -1);
+	int bv = encoding_of(v[1], override > 0);
 	if (bv < 0)
-	    return errh->warning("'%s' has no encoding, ignoring ligature", v[1].c_str());
-	int cv = encoding_of(v[3]);
+	    return (override > 0 ? errh->warning("'%s' has no encoding, ignoring ligature", v[1].c_str()) : -1);
+	int cv = encoding_of(v[3], override > 0);
 	if (cv < 0)
-	    return errh->warning("'%s' has no encoding, ignoring ligature", v[3].c_str());
+	    return (override > 0 ? errh->warning("'%s' has no encoding, ignoring ligature", v[3].c_str()) : -1);
 	Ligature lig = { av, bv, op, cv };
-	_lig.push_back(lig);
+	Ligature *what = std::find(_lig.begin(), _lig.end(), lig);
+	if (override > 0 && what < _lig.end())
+	    *what = lig;
+	else if (what == _lig.end())
+	    _lig.push_back(lig);
 	return 0;
     } else
 	return errh->error("parse error in LIGKERN");
 }
 
 int
-DvipsEncoding::parse_unicoding_words(Vector<String> &v, ErrorHandler *errh)
+DvipsEncoding::parse_unicoding_words(Vector<String> &v, int override, ErrorHandler *errh)
 {
     int av;
     if (v.size() < 2 || (v[1] != "=" && v[1] != "=:" && v[1] != ":="))
@@ -351,13 +389,12 @@ DvipsEncoding::parse_unicoding_words(Vector<String> &v, ErrorHandler *errh)
 	return errh->error("target '%s' has no encoding, ignoring UNICODING", v[0].c_str());
 
     int original_size = _unicoding.size();
-    _unicoding_map.insert(v[0], original_size);
     
     if (v.size() == 2 || (v.size() == 3 && v[2] == dot_notdef))
 	/* no warnings to delete a glyph */;
     else {
-	bool more;		// some care to get all possibilities
 	for (int i = 2; i < v.size(); i++) {
+	    bool more;		// some care to get all possibilities
 	    int uni = glyphname_unicode(v[i], &more);
 	    if (uni < 0) {
 		errh->warning("can't map '%s' to Unicode", v[i].c_str());
@@ -375,11 +412,13 @@ DvipsEncoding::parse_unicoding_words(Vector<String> &v, ErrorHandler *errh)
     }
     
     _unicoding.push_back(-1);
+    if (override > 0 || _unicoding_map[v[0]] < 0)
+	_unicoding_map.insert(v[0], original_size);
     return 0;
 }
 
 int
-DvipsEncoding::parse_words(const String &s, int (DvipsEncoding::*method)(Vector<String> &, ErrorHandler *), ErrorHandler *errh)
+DvipsEncoding::parse_words(const String &s, int override, int (DvipsEncoding::*method)(Vector<String> &, int, ErrorHandler *), ErrorHandler *errh)
 {
     Vector<String> words;
     const char *data = s.data();
@@ -393,14 +432,14 @@ DvipsEncoding::parse_words(const String &s, int (DvipsEncoding::*method)(Vector<
 	if (data == first) {
 	    data++;		// step past semicolon (or harmlessly past EOS)
 	    if (words.size() > 0) {
-		(this->*method)(words, errh);
+		(this->*method)(words, override, errh);
 		words.clear();
 	    }
 	} else
 	    words.push_back(s.substring(first, data));
     }
     if (words.size() > 0)
-	(this->*method)(words, errh);
+	(this->*method)(words, override, errh);
     return 0;
 }
 
@@ -447,21 +486,21 @@ DvipsEncoding::parse(String filename, bool ignore_ligkern, bool ignore_other, Er
 	    && isspace(token[7])
 	    && !ignore_ligkern) {
 	    lerrh.set_landmark(landmark(filename, line));
-	    parse_words(token.substring(8), &DvipsEncoding::parse_ligkern_words, &lerrh);
+	    parse_words(token.substring(8), 1, &DvipsEncoding::parse_ligkern_words, &lerrh);
 	    
 	} else if (token.length() >= 9
 		   && memcmp(token.data(), "LIGKERNX", 8) == 0
 		   && isspace(token[8])
 		   && !ignore_ligkern) {
 	    lerrh.set_landmark(landmark(filename, line));
-	    parse_words(token.substring(9), &DvipsEncoding::parse_ligkern_words, &lerrh);
+	    parse_words(token.substring(9), 1, &DvipsEncoding::parse_ligkern_words, &lerrh);
 	    
 	} else if (token.length() >= 10
 		   && memcmp(token.data(), "UNICODING", 9) == 0
 		   && isspace(token[9])
 		   && !ignore_other) {
 	    lerrh.set_landmark(landmark(filename, line));
-	    parse_words(token.substring(10), &DvipsEncoding::parse_unicoding_words, &lerrh);
+	    parse_words(token.substring(10), 1, &DvipsEncoding::parse_unicoding_words, &lerrh);
 	    
 	} else if (token.length() >= 13
 		   && memcmp(token.data(), "CODINGSCHEME", 12) == 0
@@ -487,15 +526,15 @@ DvipsEncoding::parse(String filename, bool ignore_ligkern, bool ignore_other, Er
 }
 
 int
-DvipsEncoding::parse_ligkern(const String &ligkern_text, ErrorHandler *errh)
+DvipsEncoding::parse_ligkern(const String &ligkern_text, int override, ErrorHandler *errh)
 {
-    return parse_words(ligkern_text, &DvipsEncoding::parse_ligkern_words, errh);
+    return parse_words(ligkern_text, override, &DvipsEncoding::parse_ligkern_words, errh);
 }
 
 int
-DvipsEncoding::parse_unicoding(const String &unicoding_text, ErrorHandler *errh)
+DvipsEncoding::parse_unicoding(const String &unicoding_text, int override, ErrorHandler *errh)
 {
-    return parse_words(unicoding_text, &DvipsEncoding::parse_unicoding_words, errh);
+    return parse_words(unicoding_text, override, &DvipsEncoding::parse_unicoding_words, errh);
 }
 
 void
@@ -541,7 +580,7 @@ DvipsEncoding::x_unicodes(PermString chname, Vector<uint32_t> &unicodes) const
     }
 }
 
-	
+
 void
 DvipsEncoding::make_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap, Efont::Cff::Font *font, Secondary *secondary, bool literal, ErrorHandler *errh)
 {
@@ -559,7 +598,7 @@ DvipsEncoding::make_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap,
 
 	// find all Unicodes
 	Vector<uint32_t> unicodes;
-	bool unicodes_explicit = x_unicodes(chname, unicodes);
+	(void) x_unicodes(chname, unicodes);
 
 	// find first Unicode supported by the font
 	Efont::OpenType::Glyph glyph = 0;
@@ -568,39 +607,68 @@ DvipsEncoding::make_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap,
 	    if ((glyph = map_uni(*u, cmap, metrics)) > 0)
 		glyph_uni = *u;
 
+	// find named glyph, if any
+	Efont::OpenType::Glyph named_glyph = 0;
+	if (font)
+	    named_glyph = font->glyphid(chname);
+
 	// do not use a Unicode-mapped glyph if literal
 	if (literal)
-	    glyph = 0;
+	    glyph = named_glyph;
 	
-	// use named glyph if (1) name contains a dot, or (2) nothing found
-	if (font
-	    && !unicodes_explicit
-	    && (glyph <= 0
-		|| std::find(chname.begin(), chname.end(), '.') < chname.end()))
-	    if (Efont::OpenType::Glyph named_glyph = font->glyphid(chname))
-		glyph = named_glyph;
+	// If we found a glyph, maybe use its named_glyph variant.
+	if (glyph > 0 && named_glyph > 0
+	    && std::find(chname.begin(), chname.end(), '.') < chname.end())
+	    glyph = named_glyph;
 
 	// assign slot
-	if (glyph >= 0)
+	if (glyph > 0)
 	    metrics.encode(code, glyph_uni, glyph);
     }
 
     // second pass: with secondaries
-    for (int code = 0; secondary && code < _e.size(); code++)
-	if (_e[code] != dot_notdef && metrics.glyph(code) <= 0) {
-	    PermString chname = _e[code];
-
-	    // the altselector character has its own glyph name
-	    if (code == _altselector_char)
-		chname = "altselector";
+    for (int code = 0; code < _e.size(); code++) {
+	// skip already-encoded characters and .notdef
+	if (literal || metrics.glyph(code) > 0 || _e[code] == dot_notdef)
+	    continue;
 	
-	    // loop over all Unicodes
-	    Vector<uint32_t> unicodes;
-	    (void) x_unicodes(chname, unicodes);
+	PermString chname = _e[code];
+
+	// the altselector character has its own glyph name
+	if (code == _altselector_char && !literal)
+	    chname = "altselector";
+
+	// find all Unicodes
+	Vector<uint32_t> unicodes;
+	bool unicodes_explicit = x_unicodes(chname, unicodes);
+
+	// find named glyph, if any
+	Efont::OpenType::Glyph named_glyph = 0;
+	if (font)
+	    named_glyph = font->glyphid(chname);
+
+	// 1. We were not able to find the glyph using Unicode.
+	// 2. There might be a named_glyph.
+	// May need to try secondaries later.  Store this slot.
+	// Try secondaries, if there is no named_glyph, or explicit unicoding.
+	if (unicodes_explicit || named_glyph <= 0)
 	    for (uint32_t *u = unicodes.begin(); u < unicodes.end(); u++)
 		if (secondary->encode_uni(code, chname, *u, metrics, errh))
-		    break;
-	}
+		    goto encoded;
+
+	// 1. We were not able to find the glyph using Unicode or secondaries.
+	// 2. There might be a named_glyph.
+	// Use named glyph, if any.  Special case for "UNICODING foo =: ;",
+	// which should turn off the character (even if a named_glyph exists),
+	// UNLESS the glyph was explicitly requested.
+	if (named_glyph > 0
+	    && (!unicodes_explicit || unicodes.size() > 0
+		|| (_encoding_required.size() > code && _encoding_required[code])))
+	    metrics.encode(code, unicodes.size() ? unicodes[0] : 0, named_glyph);
+
+      encoded:
+	/* all set */;
+    }
 
     // final pass: complain
     for (int code = 0; code < _e.size(); code++)
@@ -611,106 +679,6 @@ DvipsEncoding::make_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap,
 }
 
 
-#if 0
-void
-DvipsEncoding::make_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap, Efont::Cff::Font *font, Secondary *secondary, ErrorHandler *errh)
-{
-    for (int i = 0; i < _e.size(); i++) {
-	PermString chname = _e[i];
-
-	// the altselector character has its own glyph name
-	if (i == _altselector_char)
-	    chname = "altselector";
-
-	// common case: skip .notdef
-	if (chname == dot_notdef)
-	    continue;
-	
-	Efont::OpenType::Glyph gid = 0;
-	
-	// check UNICODING map
-	int m = _unicoding_map[chname];
-	if (m >= 0) {
-	    // use first mapped character in the list; secondaries are allowed
-	    for (; _unicoding[m] >= 0 && gid <= 0; m++) {
-		gid = map_uni(_unicoding[m], cmap, metrics);
-		if (gid <= 0 && secondary
-		    && secondary->encode_uni(i, chname, _unicoding[m], *this, metrics, errh))
-		    goto encoded;
-	    }
-	} else {
-	    // otherwise, try to map this glyph name to Unicode
-	    bool more;
-	    if ((m = glyphname_unicode(chname, &more)) >= 0)
-		gid = map_uni(m, cmap, metrics);
-	    // might be multiple possibilities
-	    if (!gid && more) {
-		String gn = chname;
-		do {
-		    gn += String("/");
-		    if ((m = glyphname_unicode(gn, &more)) >= 0)
-			gid = map_uni(m, cmap, metrics);
-		} while (!gid && more);
-	    }
-	    // if that didn't work, try the glyph name
-	    if (!gid && font)
-		gid = font->glyphid(chname);
-	    // always try the literal glyph name if it contained a '.'
-	    if (font && std::find(chname.begin(), chname.end(), '.') < chname.end())
-		if (Efont::OpenType::Glyph gid2 = font->glyphid(chname))
-		    gid = gid2;
-	    // as a last resort, try adding it with secondary
-	    if (gid <= 0 && secondary
-		&& (m = glyphname_unicode(chname)) >= 0
-		&& secondary->encode_uni(i, chname, m, *this, metrics, errh))
-		goto encoded;
-	    // map unknown glyphs to 0
-	    if (gid < 0)
-		gid = 0;
-	}
-
-	metrics.encode(i, gid);
-	if (gid == 0)
-	    bad_codepoint(i);
-
-      encoded: ;
-    }
-    metrics.set_coding_scheme(_coding_scheme);
-}
-
-void
-DvipsEncoding::make_literal_metrics(Metrics &metrics, const Efont::OpenType::Cmap &cmap, Efont::Cff::Font *font)
-{
-    for (int i = 0; i < _e.size(); i++)
-	if (_e[i] != dot_notdef) {
-	    Efont::OpenType::Glyph gid = font->glyphid(_e[i]);
-	    if (gid < 0)
-		gid = 0;
-	    metrics.encode(i, gid);
-	    if (gid == 0)
-		bad_codepoint(i);
-	}
-    metrics.set_coding_scheme(_coding_scheme);
-}
-
-const Vector<uint32_t> &
-DvipsEncoding::unicodes() const
-{
-    if (_unicodes.size() == 0) {
-	_unicodes.assign(_e.size(), 0xFFFFFFFFU);
-	for (int i = 0; i < _e.size(); i++)
-	    if (_e[i] != dot_notdef) {
-		int m = _unicoding_map[_e[i]];
-		if (m >= 0)
-		    _unicodes[i] = _unicoding[m];
-		else if ((m = glyphname_unicode(_e[i])) >= 0)
-		    _unicodes[i] = m;
-	    }
-    }
-    return _unicodes;
-}
-#endif
-
 void
 DvipsEncoding::apply_ligkern_lig(Metrics &metrics, ErrorHandler *errh) const
 {
@@ -721,13 +689,16 @@ DvipsEncoding::apply_ligkern_lig(Metrics &metrics, ErrorHandler *errh) const
 	    /* nada */;
 	else if (l.join == J_NOLIG || l.join == J_NOLIGKERN)
 	    metrics.remove_ligatures(l.c1, l.c2);
-	else if (l.join == J_LIG)
+	else if (l.join == J_LIG) {
+	    metrics.remove_ligatures(l.c1, l.c2);
 	    metrics.add_ligature(l.c1, l.c2, l.d);
-	else if (l.join == J_LIGC)
+	} else if (l.join == J_LIGC) {
+	    metrics.remove_ligatures(l.c1, l.c2);
 	    metrics.add_ligature(l.c1, l.c2, metrics.pair_code(l.d, l.c2));
-	else if (l.join == J_CLIG)
+	} else if (l.join == J_CLIG) {
+	    metrics.remove_ligatures(l.c1, l.c2);
 	    metrics.add_ligature(l.c1, l.c2, metrics.pair_code(l.c1, l.d));
-	else {
+	} else {
 	    static int complex_join_warning = 0;
 	    if (!complex_join_warning) {
 		errh->warning("complex LIGKERN ligature removed (I only support '=:', '=:|', and '|=:')");
