@@ -1,6 +1,6 @@
 /* otftotfm.cc -- driver for translating OpenType fonts to TeX metrics
  *
- * Copyright (c) 2003-2004 Eddie Kohler
+ * Copyright (c) 2003-2005 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -14,6 +14,9 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
+#ifdef WIN32
+# define _USE_MATH_DEFINES
+#endif
 #include <efont/psres.hh>
 #include <efont/t1rw.hh>
 #include <efont/t1font.hh>
@@ -79,10 +82,18 @@ using namespace Efont;
 #define ALTSELECTOR_CHAR_OPT	324
 #define INCLUDE_ALTERNATES_OPT	325
 #define EXCLUDE_ALTERNATES_OPT	326
-#define ALTSELECTOR_FEATURE_OPT	327
-#define DEFAULT_LIGKERN_OPT	328
-#define NO_ECOMMAND_OPT		329
-#define LETTER_FEATURE_OPT	330
+#define CLEAR_ALTERNATES_OPT	327
+#define ALTSELECTOR_FEATURE_OPT	328
+#define DEFAULT_LIGKERN_OPT	329
+#define NO_ECOMMAND_OPT		330
+#define LETTER_FEATURE_OPT	331
+#define INCLUDE_SUBS_OPT	332
+#define EXCLUDE_SUBS_OPT	333
+#define CLEAR_SUBS_OPT		334
+#define SUBS_FILTER_OPT		335
+#define ALTERNATES_FILTER_OPT	336
+#define SPACE_FACTOR_OPT	337
+#define MATH_SPACING_OPT	338
 
 #define AUTOMATIC_OPT		341
 #define FONT_NAME_OPT		342
@@ -98,6 +109,7 @@ using namespace Efont;
 #define PL_OPT			352
 #define TFM_OPT			353
 #define MAP_FILE_OPT		354
+#define OUTPUT_ENCODING_OPT	355
 
 #define DIR_OPTS		360
 #define ENCODING_DIR_OPT	(DIR_OPTS + O_ENCODING)
@@ -111,6 +123,7 @@ using namespace Efont;
 #define NO_ENCODING_OPT		(NO_OUTPUT_OPTS + G_ENCODING)
 #define NO_TYPE1_OPT		(NO_OUTPUT_OPTS + G_TYPE1)
 #define NO_DOTLESSJ_OPT		(NO_OUTPUT_OPTS + G_DOTLESSJ)
+#define NO_UPDMAP_OPT		(NO_OUTPUT_OPTS + G_UPDMAP)
 
 #define CHAR_OPTTYPE		(Clp_FirstUserType)
 
@@ -120,6 +133,11 @@ static Clp_Option options[] = {
     { "feature", 'f', FEATURE_OPT, Clp_ArgString, 0 },
     { "letter-feature", 0, LETTER_FEATURE_OPT, Clp_ArgString, 0 },
     { "lf", 0, LETTER_FEATURE_OPT, Clp_ArgString, 0 },
+    { "include-substitutions", 0, INCLUDE_SUBS_OPT, Clp_ArgString, 0 },
+    { "exclude-substitutions", 0, EXCLUDE_SUBS_OPT, Clp_ArgString, 0 },
+    { "clear-substitutions", 0, CLEAR_SUBS_OPT, 0, 0 },
+    { "substitution-filter", 0, SUBS_FILTER_OPT, Clp_ArgString, 0 },
+    { "subs-filter", 0, SUBS_FILTER_OPT, Clp_ArgString, 0 },
     { "encoding", 'e', ENCODING_OPT, Clp_ArgString, 0 },
     { "literal-encoding", 0, LITERAL_ENCODING_OPT, Clp_ArgString, 0 },
     { "extend", 'E', EXTEND_OPT, Clp_ArgDouble, 0 },
@@ -131,7 +149,7 @@ static Clp_Option options[] = {
     { "kern-precision", 'k', MINIMUM_KERN_OPT, Clp_ArgDouble, 0 },
     { "ligkern", 0, LIGKERN_OPT, Clp_ArgString, 0 },
     { "no-encoding-commands", 0, NO_ECOMMAND_OPT, 0, Clp_OnlyNegated },
-    { "default-ligkern", 0, DEFAULT_LIGKERN_OPT, 0, 0 },
+    { "default-ligkern", 0, DEFAULT_LIGKERN_OPT, 0, Clp_Negate },
     { "unicoding", 0, UNICODING_OPT, Clp_ArgString, 0 },
     { "coding-scheme", 0, CODINGSCHEME_OPT, Clp_ArgString, 0 },
     { "boundary-char", 0, BOUNDARY_CHAR_OPT, CHAR_OPTTYPE, 0 },
@@ -141,13 +159,19 @@ static Clp_Option options[] = {
     { "design-size", 0, DESIGN_SIZE_OPT, Clp_ArgDouble, 0 },
     { "include-alternates", 0, INCLUDE_ALTERNATES_OPT, Clp_ArgString, 0 },
     { "exclude-alternates", 0, EXCLUDE_ALTERNATES_OPT, Clp_ArgString, 0 },
+    { "clear-alternates", 0, CLEAR_ALTERNATES_OPT, 0, 0 },
+    { "alternates-filter", 0, ALTERNATES_FILTER_OPT, Clp_ArgString, 0 },
+    { "space-factor", 0, SPACE_FACTOR_OPT, Clp_ArgDouble, 0 },
+    { "math-spacing", 0, MATH_SPACING_OPT, 0, Clp_Negate },
     
     { "pl", 'p', PL_OPT, 0, 0 },
     { "virtual", 0, VIRTUAL_OPT, 0, Clp_Negate },
     { "no-encoding", 0, NO_ENCODING_OPT, 0, Clp_OnlyNegated },
     { "no-type1", 0, NO_TYPE1_OPT, 0, Clp_OnlyNegated },
     { "no-dotlessj", 0, NO_DOTLESSJ_OPT, 0, Clp_OnlyNegated },
+    { "no-updmap", 0, NO_UPDMAP_OPT, 0, Clp_OnlyNegated },
     { "map-file", 0, MAP_FILE_OPT, Clp_ArgString, Clp_Negate },
+    { "output-encoding", 0, OUTPUT_ENCODING_OPT, Clp_ArgString, Clp_Optional },
         
     { "automatic", 'a', AUTOMATIC_OPT, 0, Clp_Negate },
     { "name", 'n', FONT_NAME_OPT, Clp_ArgString, 0 },
@@ -199,9 +223,8 @@ static Vector<Efont::OpenType::Tag> interesting_scripts;
 static Vector<Efont::OpenType::Tag> interesting_features;
 static Vector<Efont::OpenType::Tag> altselector_features;
 static GlyphFilter null_filter;
-static HashMap<Efont::OpenType::Tag, GlyphFilter*> feature_filters(&null_filter);
-
-static GlyphFilter alternate_filter;
+static HashMap<Efont::OpenType::Tag, GlyphFilter*> feature_filters(0);
+static HashMap<Efont::OpenType::Tag, GlyphFilter*> altselector_feature_filters(0);
 
 static String font_name;
 static String encoding_file;
@@ -210,11 +233,13 @@ static double slant;
 static int letterspace;
 static double design_size;
 static double minimum_kern = 2.0;
+static double space_factor = 1.0;
+static bool math_spacing = false;
 
 static String out_encoding_file;
 static String out_encoding_name;
 
-int output_flags = G_ENCODING | G_METRICS | G_VMETRICS | G_PSFONTSMAP | G_TYPE1 | G_DOTLESSJ | G_BINARY;
+int output_flags = G_ENCODING | G_METRICS | G_VMETRICS | G_PSFONTSMAP | G_TYPE1 | G_DOTLESSJ | G_UPDMAP | G_BINARY;
 
 bool automatic = false;
 bool verbose = false;
@@ -222,6 +247,8 @@ bool no_create = false;
 bool quiet = false;
 bool force = false;
 
+static String otf_data;
+
 
 void
 usage_error(ErrorHandler *errh, const char *error_message, ...)
@@ -254,12 +281,19 @@ Font feature and transformation options:\n\
   -s, --script=SCRIPT[.LANG]   Use features for script SCRIPT[.LANG] [latn].\n\
   -f, --feature=FEAT           Activate feature FEAT.\n\
   --lf, --letter-feature=FEAT  Activate feature FEAT for letters.\n\
+      --subs-filter=PAT        Substitute only characters matching PAT.\n\
+      --include-subs=PAT       Same, but cumulative.\n\
+      --exclude-subs=PAT       Don't substitute characters matching PAT.\n\
+      --clear-subs             Clear included/excluded substitutions.\n\
   -E, --extend=F               Widen characters by a factor of F.\n\
   -S, --slant=AMT              Oblique characters by AMT, generally <<1.\n\
   -L, --letterspacing=AMT      Letterspace each character by AMT units.\n\
+      --math-spacing           Use letterspacing appropriate for math.\n\
   -k, --min-kern=N             Omit kerns with absolute value < N [2.0].\n\
+      --space-factor=F         Scale wordspace by a factor of F.\n\
       --design-size=SIZE       Set font design size to SIZE.\n\
-\n\
+\n");
+    printf("\
 Encoding options:\n\
   -e, --encoding=FILE          Use DVIPS encoding FILE as a base encoding.\n\
       --literal-encoding=FILE  Use DVIPS encoding FILE verbatim.\n\
@@ -271,8 +305,10 @@ Encoding options:\n\
       --boundary-char=CHAR     Set the boundary character to CHAR.\n\
       --altselector-char=CHAR  Set the alternate selector character to CHAR.\n\
       --altselector-feature=F  Activate feature F for --altselector-char.\n\
+      --alternates-filter=PAT  Include only alternate characters matching PAT.\n\
+      --include-alternates=PAT Same, but cumulative.\n\
       --exclude-alternates=PAT Ignore alternate characters matching PAT.\n\
-      --include-alternates=PAT Include only alternate characters matching PAT.\n\
+      --clear-alternates       Clear included/excluded alternates.\n\
 \n");
     printf("\
 Automatic mode options:\n\
@@ -281,6 +317,7 @@ Automatic mode options:\n\
       --typeface=NAME          Set typeface name for TDS [<font family>].\n\
       --no-type1               Do not generate Type 1 fonts.\n\
       --no-dotlessj            Do not generate dotless-j fonts.\n\
+      --no-updmap              Do not run updmap.\n\
 \n\
 Output options:\n\
   -n, --name=NAME              Generated font name is NAME.\n\
@@ -288,6 +325,7 @@ Output options:\n\
       --no-virtual             Do not generate VFs or VPLs.\n\
       --no-encoding            Do not generate an encoding file.\n\
       --no-map                 Do not generate a psfonts.map line.\n\
+      --output-encoding[=FILE] Only generate an encoding file.\n\
 \n");
     printf("\
 File location options:\n\
@@ -311,9 +349,10 @@ Other options:\n\
   -q, --quiet                  Do not generate any error messages.\n\
       --version                Print version number and exit.\n\
 \n\
-Report bugs to <kohler@icir.org>.\n");
+Report bugs to <kohler@cs.ucla.edu>.\n");
 }
 
+#ifndef WIN32
 extern "C" {
 static void
 sigchld_handler(int s)
@@ -344,6 +383,7 @@ handle_sigchld()
     signal(sigchld, sigchld_handler);
 #endif
 }
+#endif
 
 
 // MAIN
@@ -473,7 +513,7 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
     double du = (design_units == 1000 ? 1. : design_units / 1000.);
     
     double val;
-    (void) family_cff->dict_value(Efont::Cff::oItalicAngle, 0, &val);
+    (void) family_cff->dict_value(Efont::Cff::oItalicAngle, &val);
     double actual_slant = -tan(val * M_PI / 180.0) + slant;
     if (actual_slant)
 	fprintf(f, "   (SLANT R %g)\n", actual_slant);
@@ -481,18 +521,18 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
     OpenType::Cmap cmap(family_otf.table("cmap"));
     
     if (char_bounds(bounds, width, family_cff, cmap, ' ', font_xform)) {
-	// advance space width by letterspacing
-	width += letterspace;
-	fprint_real(f, "   (SPACE", width, du);
-	if (family_cff->dict_value(Efont::Cff::oIsFixedPitch, 0, &val) && val) {
+	// advance space width by letterspacing, scale by space_factor
+	double space_width = (width + letterspace) * space_factor;
+	fprint_real(f, "   (SPACE", space_width, du);
+	if (family_cff->dict_value(Efont::Cff::oIsFixedPitch, &val) && val) {
 	    // fixed-pitch: no space stretch or shrink
 	    fprint_real(f, "   (STRETCH", 0, du);
 	    fprint_real(f, "   (SHRINK", 0, du);
-	    fprint_real(f, "   (EXTRASPACE", width, du);
+	    fprint_real(f, "   (EXTRASPACE", space_width, du);
 	} else {
-	    fprint_real(f, "   (STRETCH", width / 2., du);
-	    fprint_real(f, "   (SHRINK", width / 3., du);
-	    fprint_real(f, "   (EXTRASPACE", width / 6., du);
+	    fprint_real(f, "   (STRETCH", space_width / 2., du);
+	    fprint_real(f, "   (SHRINK", space_width / 3., du);
+	    fprint_real(f, "   (EXTRASPACE", space_width / 6., du);
 	}
     }
 
@@ -560,7 +600,7 @@ output_pl(const Metrics &metrics, const String &ps_name, int boundary_char,
     // don't print KRN x after printing LIG x
     uint32_t used[8];
     for (int i = 0; i <= 256; i++)
-	if (metrics.glyph(i)) {
+	if (metrics.glyph(i) && minimum_kern < 10000) {
 	    int any_lig = metrics.ligatures(i, lig_code2, lig_outcode, lig_context);
 	    int any_kern = metrics.kerns(i, kern_code2, kern_amt);
 	    if (any_lig || any_kern) {
@@ -744,7 +784,7 @@ write_encoding_file(String &filename, const String &encoding_name,
 	lock.l_start = 0;
 	lock.l_len = 0;
 	int result;
-	while ((result = fcntl(fd, F_SETLKW, &lock)) < 0 || errno == EINTR)
+	while ((result = fcntl(fd, F_SETLKW, &lock)) < 0 && errno == EINTR)
 	    /* try again */;
 	if (result < 0) {
 	    result = errno;
@@ -802,7 +842,7 @@ write_encoding_file(String &filename, const String &encoding_name,
 	update_odir(O_ENCODING, filename, errh);
     return 0;
 }
-	
+
 static void
 output_encoding(const Metrics &metrics,
 		const Vector<PermString> &glyph_names,
@@ -837,7 +877,9 @@ output_encoding(const Metrics &metrics,
     out_encoding_name = "AutoEnc_" + String(text_digest);
 
     // create encoding filename
-    out_encoding_file = getodir(O_ENCODING, errh) + String("/a_") + String(text_digest).substring(0, 6) + ".enc";
+    bool output_encoding_only = (bool) out_encoding_file;
+    if (!out_encoding_file)
+	out_encoding_file = getodir(O_ENCODING, errh) + String("/a_") + String(text_digest).substring(0, 6) + ".enc";
 
     // exit if we're not responsible for generating an encoding
     if (!(output_flags & G_ENCODING))
@@ -847,9 +889,10 @@ output_encoding(const Metrics &metrics,
     // 3.Jun.2003: stick command line definition at the end of the encoding,
     // where it won't confuse idiotic ps2pk
     StringAccum contents;
-    contents << "% THIS FILE WAS AUTOMATICALLY GENERATED -- DO NOT EDIT\n\n\
-%%" << out_encoding_name << "\n\
-% Encoding created by otftotfm" << current_time << "\n\
+    if (!output_encoding_only)
+	contents << "% THIS FILE WAS AUTOMATICALLY GENERATED -- DO NOT EDIT\n\n\
+%%" << out_encoding_name << "\n";
+    contents << "% Encoding created by otftotfm" << current_time << "\n\
 % Command line follows encoding\n";
     
     // the encoding itself
@@ -881,47 +924,12 @@ output_encoding(const Metrics &metrics,
     }
     
     // open encoding file
-    if (write_encoding_file(out_encoding_file, out_encoding_name, contents, errh) == 1)
+    if (out_encoding_file == "-")
+	fwrite(contents.data(), 1, contents.length(), stdout);
+    else if (write_encoding_file(out_encoding_file, out_encoding_name, contents, errh) == 1)
 	update_odir(O_ENCODING, out_encoding_file, errh);
 }
 
-static int
-temporary_file(String &filename, ErrorHandler *errh)
-{
-    if (no_create)
-	return 0;		// random number suffices
-    
-#ifdef HAVE_MKSTEMP
-    const char *tmpdir = getenv("TMPDIR");
-    if (tmpdir)
-	filename = String(tmpdir) + "/otftotfm.XXXXXX";
-    else {
-# ifdef P_tmpdir
-	filename = P_tmpdir "/otftotfm.XXXXXX";
-# else
-	filename = "/tmp/otftotfm.XXXXXX";
-# endif
-    }
-    int fd = mkstemp(filename.mutable_c_str());
-    if (fd < 0)
-	errh->error("temporary file '%s': %s", filename.c_str(), strerror(errno));
-    return fd;
-#else  // !HAVE_MKSTEMP
-    for (int tries = 0; tries < 5; tries++) {
-	if (!(filename = tmpnam(0)))
-	    return errh->error("cannot create temporary file");
-# ifdef O_EXCL
-	int fd = ::open(filename.c_str(), O_RDWR | O_CREAT | O_EXCL | O_TRUNC, 0600);
-# else
-	int fd = ::open(filename.c_str(), O_RDWR | O_CREAT | O_TRUNC, 0600);
-# endif
-	if (fd >= 0)
-	    return fd;
-    }
-    return errh->error("temporary file '%s': %s", filename.c_str(), strerror(errno));
-#endif
-}
-
 static void
 output_tfm(const Metrics &metrics, const String &ps_name, int boundary_char,
 	   const OpenType::Font &family_otf, const Cff::Font *family_cff,
@@ -947,9 +955,9 @@ output_tfm(const Metrics &metrics, const String &ps_name, int boundary_char,
 
     StringAccum command;
     if (vpl)
-	command << "vptovf " << pl_filename << ' ' << vf_filename << ' ' << tfm_filename;
+	command << "vptovf " << shell_quote(pl_filename) << ' ' << shell_quote(vf_filename) << ' ' << shell_quote(tfm_filename) << " >&2";
     else
-	command << "pltotf " << pl_filename << ' ' << tfm_filename;
+	command << "pltotf " << shell_quote(pl_filename) << ' ' << shell_quote(tfm_filename) << " >&2";
     
     int status = mysystem(command.c_str(), errh);
 
@@ -978,7 +986,7 @@ output_metrics(Metrics &metrics, const String &ps_name, int boundary_char,
     if (need_virtual) {
 	if (output_flags & G_VMETRICS)
 	    base_font_name = make_base_font_name(font_name);
-	else
+	else if (output_flags & G_METRICS)
 	    errh->warning("features require virtual fonts");
     }
     
@@ -1010,6 +1018,9 @@ output_metrics(Metrics &metrics, const String &ps_name, int boundary_char,
     }
 
     // output metrics
+    double save_minimum_kern = minimum_kern;
+    if (need_virtual)
+	minimum_kern = 100000;
     if (!(output_flags & G_METRICS))
 	/* do nothing */;
     else if (output_flags & G_BINARY) {
@@ -1020,6 +1031,7 @@ output_metrics(Metrics &metrics, const String &ps_name, int boundary_char,
 	output_pl(metrics, ps_name, boundary_char, family_otf, family_cff, false, outfile, errh);
 	update_odir(O_PL, outfile, errh);
     }
+    minimum_kern = save_minimum_kern;
 
     // print DVIPS map line
     if (errh->nerrors() == 0 && (output_flags & G_PSFONTSMAP)) {
@@ -1102,7 +1114,7 @@ report_underused_features(const HashMap<uint32_t, int> &feature_usage, ErrorHand
 		sa << *a << sep;
 	    sa << (x[i].size() > 1 ? "and " : "") << x[i].back()
 	       << (x[i].size() > 1 ? " features" : " feature") << (msg_pct+1);
-	    sa.append_break_lines(sa.take_string(), 61);
+	    sa.append_break_lines(sa.take_string(), 58);
 	    errh->warning("%s", sa.c_str());
 	}
 }
@@ -1112,7 +1124,7 @@ static String otf_filename;
 static void
 main_dvips_map(const String &ps_name, StringAccum &sa, ErrorHandler *errh)
 {
-    if (String fn = installed_type1(otf_filename, ps_name, (output_flags & G_TYPE1), errh))
+    if (String fn = installed_type1(otf_filename, ps_name, (output_flags & G_TYPE1) != 0, errh))
 	sa << "<" << pathname_filename(fn);
     else
 	sa << "<<" << pathname_filename(otf_filename);
@@ -1122,7 +1134,7 @@ static void
 do_gsub(Metrics& metrics, const OpenType::Font& otf, DvipsEncoding& dvipsenc, bool dvipsenc_literal, HashMap<uint32_t, int>& feature_usage, const Vector<PermString>& glyph_names, ErrorHandler* errh)
 {
     // apply activated GSUB features
-    OpenType::Gsub gsub(otf.table("GSUB"), errh);
+    OpenType::Gsub gsub(otf.table("GSUB"), &otf, errh);
     Vector<Lookup> lookups(gsub.nlookups(), Lookup());
     find_lookups(gsub.script_list(), gsub.feature_list(), lookups, errh);
     Vector<OpenType::Substitution> subs;
@@ -1152,7 +1164,7 @@ do_gsub(Metrics& metrics, const OpenType::Font& otf, DvipsEncoding& dvipsenc, bo
 	    //for (int subno = 0; subno < subs.size(); subno++) fprintf(stderr, "%5d\t%s\n", i, subs[subno].unparse().c_str());
 
 	    // figure out which glyph filter to use
-	    int nunderstood = metrics.apply(subs, !dvipsenc_literal, i, *lookups[i].filter + alternate_filter, glyph_names);
+	    int nunderstood = metrics.apply(subs, !dvipsenc_literal, i, *lookups[i].filter, glyph_names);
 
 	    // mark as used
 	    int d = (understood && nunderstood == subs.size() ? F_GSUB_ALL : (nunderstood ? F_GSUB_PART : 0)) + F_GSUB_TRY;
@@ -1160,15 +1172,18 @@ do_gsub(Metrics& metrics, const OpenType::Font& otf, DvipsEncoding& dvipsenc, bo
 		feature_usage.find_force(lookups[i].features[j].value()) |= d;
 	}
 
-    // apply 'aalt' feature if we have variant selectors
+    // apply alternate selectors
     if (metrics.altselectors() && !dvipsenc_literal) {
 	// apply default features
 	if (!altselector_features.size()) {
 	    altselector_features.push_back(OpenType::Tag("dlig"));
+	    altselector_feature_filters.insert(OpenType::Tag("dlig"), &null_filter);
 	    altselector_features.push_back(OpenType::Tag("salt"));
+	    altselector_feature_filters.insert(OpenType::Tag("salt"), &null_filter);
 	}
 	// do lookups
 	altselector_features.swap(interesting_features);
+	altselector_feature_filters.swap(feature_filters);
 	Vector<Lookup> alt_lookups(gsub.nlookups(), Lookup());
 	find_lookups(gsub.script_list(), gsub.feature_list(), alt_lookups, ErrorHandler::silent_handler());
 	Vector<OpenType::Substitution> alt_subs;
@@ -1177,9 +1192,10 @@ do_gsub(Metrics& metrics, const OpenType::Font& otf, DvipsEncoding& dvipsenc, bo
 		OpenType::GsubLookup l = gsub.lookup(i);
 		alt_subs.clear();
 		(void) l.unparse_automatics(gsub, alt_subs);
-		metrics.apply_alternates(alt_subs, i, alternate_filter, glyph_names);
+		metrics.apply_alternates(alt_subs, i, *alt_lookups[i].filter, glyph_names);
 	    }
 	altselector_features.swap(interesting_features);
+	altselector_feature_filters.swap(feature_filters);
     }
 }
 
@@ -1293,12 +1309,13 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
     // remove extra characters
     metrics.cut_encoding(257);
     //metrics.unparse();
+    int boundary_char = dvipsenc.boundary_char();
 
     // apply letterspacing, if any
     if (letterspace) {
 	for (int code = 0; code < 256; code++) {
 	    int g = metrics.glyph(code);
-	    if (g != 0 && g != Metrics::VIRTUAL_GLYPH && code != dvipsenc.boundary_char()) {
+	    if (g != 0 && g != Metrics::VIRTUAL_GLYPH && code != boundary_char) {
 		metrics.add_single_positioning(code, letterspace / 2, 0, letterspace);
 		metrics.add_kern(code, 256, -letterspace / 2);
 		metrics.add_kern(256, code, -letterspace / 2);
@@ -1306,8 +1323,24 @@ do_file(const String &otf_filename, const OpenType::Font &otf,
 	}
     }
 
+    // apply math letterspacing, if any
+    if (math_spacing) {
+	Transform font_xform;
+	CharstringBounds boundser(font_xform);
+	int bounds[4], width;
+	for (int code = 0; code < 256; code++) {
+	    int g = metrics.glyph(code);
+	    if (g != 0 && g != Metrics::VIRTUAL_GLYPH && code != boundary_char) {
+		if (char_bounds(bounds, width, font, cmap, code, font_xform)) {
+		    int left_sb = (bounds[0] < 0 ? -bounds[0] : 0);
+		    int right_sb = (bounds[2] > width ? bounds[2] - width : 0);
+		    metrics.add_single_positioning(code, left_sb, 0, left_sb + right_sb);
+		}
+	    }
+	}
+    }
+
     // reencode right components of boundary_glyph as boundary_char
-    int boundary_char = dvipsenc.boundary_char();
     if (metrics.reencode_right_ligkern(256, boundary_char) > 0
 	&& boundary_char < 0) {
 	errh->warning("no boundary character, ignoring some ligatures and/or kerns\n");
@@ -1386,7 +1419,9 @@ clp_parse_char(Clp_Parser *clp, const char *arg, int complain, void *)
 int
 main(int argc, char *argv[])
 {
+#ifndef WIN32
     handle_sigchld();
+#endif
     String::static_initialize();
     Clp_Parser *clp =
 	Clp_NewParser(argc, (const char * const *)argv, sizeof(options) / sizeof(options[0]), options);
@@ -1411,8 +1446,12 @@ main(int argc, char *argv[])
     bool literal_encoding = false;
     Vector<String> ligkern;
     Vector<String> unicoding;
-    bool no_ecommand = false, default_ligkern = false;
+    bool no_ecommand = false, default_ligkern = true;
     String codingscheme;
+
+    GlyphFilter current_substitution_filter;
+    GlyphFilter current_alternate_filter;
+    GlyphFilter* current_filter_ptr = &null_filter;
     
     while (1) {
 	int opt = Clp_Next(clp);
@@ -1439,25 +1478,49 @@ main(int argc, char *argv[])
 
 	  case FEATURE_OPT: {
 	      OpenType::Tag t(clp->arg);
-	      if (t.valid())
-		  interesting_features.push_back(t);
-	      else
+	      if (!t.valid())
 		  usage_error(errh, "bad feature tag");
+	      else if (feature_filters[t])
+		  usage_error(errh, "feature '%s' included twice", t.text().c_str());
+	      else {
+		  if (!current_filter_ptr)
+		      current_filter_ptr = new GlyphFilter(current_substitution_filter + current_alternate_filter);
+		  interesting_features.push_back(t);
+		  feature_filters.insert(t, current_filter_ptr);
+	      }
 	      break;
 	  }
       
 	  case LETTER_FEATURE_OPT: {
 	      OpenType::Tag t(clp->arg);
-	      if (t.valid()) {
+	      if (!t.valid())
+		  usage_error(errh, "bad feature tag");
+	      else if (feature_filters[t])
+		  usage_error(errh, "feature '%s' included twice", t.text().c_str());
+	      else {
 		  interesting_features.push_back(t);
 		  GlyphFilter* gf = new GlyphFilter;
 		  gf->add_substitution_filter("<Letter>", false, errh);
+		  *gf += current_alternate_filter;
 		  feature_filters.insert(t, gf); 
-	      } else
-		  usage_error(errh, "bad feature tag");
+	      }
 	      break;
 	  }
 
+	  case SUBS_FILTER_OPT:
+	    current_substitution_filter = null_filter;
+	    /* fallthru */
+	  case EXCLUDE_SUBS_OPT:
+	  case INCLUDE_SUBS_OPT:
+	    current_substitution_filter.add_substitution_filter(clp->arg, opt == EXCLUDE_SUBS_OPT, errh);
+	    current_filter_ptr = 0;
+	    break;
+
+	  case CLEAR_SUBS_OPT:
+	    current_substitution_filter = null_filter;
+	    current_filter_ptr = 0;
+	    break;
+	    
 	  case ENCODING_OPT:
 	    if (encoding_file)
 		usage_error(errh, "encoding specified twice");
@@ -1489,6 +1552,16 @@ main(int argc, char *argv[])
 	    letterspace = clp->val.i;
 	    break;
 
+	  case SPACE_FACTOR_OPT:
+	    if (space_factor != 1)
+		usage_error(errh, "space factor specified twice");
+	    space_factor = clp->val.d;
+	    break;
+
+	  case MATH_SPACING_OPT:
+	    math_spacing = !clp->negated;
+	    break;
+	    
 	  case DESIGN_SIZE_OPT:
 	    if (design_size > 0)
 		usage_error(errh, "design size value specified twice");
@@ -1506,7 +1579,7 @@ main(int argc, char *argv[])
 	    break;
 
 	  case DEFAULT_LIGKERN_OPT:
-	    default_ligkern = true;
+	    default_ligkern = !clp->negated;
 	    break;
 
 	  case BOUNDARY_CHAR_OPT:
@@ -1519,29 +1592,32 @@ main(int argc, char *argv[])
 
 	  case ALTSELECTOR_FEATURE_OPT: {
 	      OpenType::Tag t(clp->arg);
-	      if (t.valid())
-		  altselector_features.push_back(t);
-	      else
+	      if (!t.valid())
 		  usage_error(errh, "bad feature tag");
-	      break;
-	  }
-
-	  case EXCLUDE_ALTERNATES_OPT:
-	  case INCLUDE_ALTERNATES_OPT: {
-	      const char *s = clp->arg;
-	      while (*s) {
-		  const char *start = s;
-		  while (*s && !isspace(*s))
-		      s++;
-		  if (s > start) {
-		      String str(start, s - start);
-		      alternate_filter.add_alternate_filter(str, opt == EXCLUDE_ALTERNATES_OPT, errh);
-		  }
-		  while (isspace(*s))
-		      s++;
+	      else if (altselector_feature_filters[t])
+		  usage_error(errh, "altselector feature '%s' included twice", t.text().c_str());
+	      else {
+		  if (!current_filter_ptr)
+		      current_filter_ptr = new GlyphFilter(current_substitution_filter + current_alternate_filter);
+		  altselector_features.push_back(t);
+		  altselector_feature_filters.insert(t, current_filter_ptr);
 	      }
 	      break;
 	  }
+
+	  case ALTERNATES_FILTER_OPT:
+	    current_alternate_filter = null_filter;
+	    /* fallthru */
+	  case EXCLUDE_ALTERNATES_OPT:
+	  case INCLUDE_ALTERNATES_OPT:
+	    current_alternate_filter.add_alternate_filter(clp->arg, opt == EXCLUDE_ALTERNATES_OPT, errh);
+	    current_filter_ptr = 0;
+	    break;
+
+	  case CLEAR_ALTERNATES_OPT:
+	    current_alternate_filter = null_filter;
+	    current_filter_ptr = 0;
+	    break;
 	    
 	  case UNICODING_OPT:
 	    unicoding.push_back(clp->arg);
@@ -1581,9 +1657,17 @@ main(int argc, char *argv[])
 	  case NO_ENCODING_OPT:
 	  case NO_TYPE1_OPT:
 	  case NO_DOTLESSJ_OPT:
+	  case NO_UPDMAP_OPT:
 	    output_flags &= ~(opt - NO_OUTPUT_OPTS);
 	    break;
 
+	  case OUTPUT_ENCODING_OPT:
+	    if (out_encoding_file)
+		usage_error(errh, "encoding output file specified twice");
+	    out_encoding_file = (clp->have_arg ? clp->arg : "-");
+	    output_flags = G_ENCODING;
+	    break;
+
 	  case MINIMUM_KERN_OPT:
 	    minimum_kern = clp->val.d;
 	    break;
@@ -1664,7 +1748,7 @@ main(int argc, char *argv[])
 
 	  case VERSION_OPT:
 	    printf("otftotfm (LCDF typetools) %s\n", VERSION);
-	    printf("Copyright (C) 2002-2004 Eddie Kohler\n\
+	    printf("Copyright (C) 2002-2005 Eddie Kohler\n\
 This is free software; see the source for copying conditions.\n\
 There is NO warranty, not even for merchantability or fitness for a\n\
 particular purpose.\n");
@@ -1709,14 +1793,14 @@ particular purpose.\n");
 	    encoding_file = "";
     
 	// read font
-	String font_data = read_file(input_file, errh);
+	otf_data = read_file(input_file, errh);
 	if (errh->nerrors())
 	    exit(1);
 
 	LandmarkErrorHandler cerrh(errh, printable_filename(input_file));
 	BailErrorHandler bail_errh(&cerrh);
 
-	OpenType::Font otf(font_data, &bail_errh);
+	OpenType::Font otf(otf_data, &bail_errh);
 	assert(otf.ok());
 
 	// figure out scripts we care about
@@ -1725,6 +1809,7 @@ particular purpose.\n");
 	    interesting_scripts.push_back(Efont::OpenType::Tag());
 	}
 	std::sort(interesting_features.begin(), interesting_features.end());
+	std::sort(altselector_features.begin(), altselector_features.end());
 
 	// read glyphlist
 	if (String s = read_file(glyphlist_file, errh, true))
@@ -1734,7 +1819,7 @@ particular purpose.\n");
 	DvipsEncoding dvipsenc;
 	if (encoding_file) {
 	    if (String path = locate_encoding(encoding_file, errh))
-		dvipsenc.parse(path, no_ecommand || default_ligkern, no_ecommand, errh);
+		dvipsenc.parse(path, no_ecommand, no_ecommand, errh);
 	    else
 		errh->fatal("encoding '%s' not found", encoding_file.c_str());
 	} else {
@@ -1751,16 +1836,16 @@ particular purpose.\n");
 	}
 
 	// apply default ligkern commands
-	if (default_ligkern || (!dvipsenc.file_had_ligkern() && !ligkern.size() && !no_ecommand))
-	    dvipsenc.parse_ligkern(default_ligkerns, ErrorHandler::silent_handler());
+	if (default_ligkern)
+	    dvipsenc.parse_ligkern(default_ligkerns, 0, ErrorHandler::silent_handler());
     
 	// apply command-line ligkern commands and coding scheme
 	cerrh.set_landmark("--ligkern command");
 	for (int i = 0; i < ligkern.size(); i++)
-	    dvipsenc.parse_ligkern(ligkern[i], &cerrh);
+	    dvipsenc.parse_ligkern(ligkern[i], 1, &cerrh);
 	cerrh.set_landmark("--unicoding command");
 	for (int i = 0; i < unicoding.size(); i++)
-	    dvipsenc.parse_unicoding(unicoding[i], &cerrh);
+	    dvipsenc.parse_unicoding(unicoding[i], 1, &cerrh);
 	if (codingscheme)
 	    dvipsenc.set_coding_scheme(codingscheme);
 
