@@ -18,13 +18,19 @@
 #include <lcdf/error.hh>
 #include <lcdf/straccum.hh>
 #include <lcdf/vector.hh>
+#include <ctype.h>
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
-#if defined(_MSDOS) || defined(_WIN32)
+#ifdef HAVE_FCNTL_H
 # include <fcntl.h>
+#endif
+#if defined(_MSDOS) || defined(_WIN32)
 # include <io.h>
 #endif
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 
 String
 read_file(String filename, ErrorHandler *errh, bool warning)
@@ -76,6 +82,36 @@ pathname_filename(const String &path)
 	return path;
 }
 
+String
+shell_quote(const String &str)
+{
+#if defined(_MSDOS) || defined(_WIN32)
+    return str;			// XXX
+#else
+    if (!str)
+	return String::stable_string("''");
+
+    const char *begin = str.begin();
+    const char *end = str.end();
+    StringAccum sa;
+    for (const char *s = begin; s < end; s++)
+	if (isalnum((unsigned char) *s) || *s == '_' || *s == '-' || *s == '+' || *s == '/' || *s == ':' || *s == '.')
+	    /* do nothing */;
+	else {
+	    sa.append(begin, s);
+	    sa.append('\\');
+	    begin = s;
+	}
+
+    if (!sa)
+	return str;
+    else {
+	sa.append(begin, end);
+	return sa.take_string();
+    }
+#endif
+}
+
 int
 mysystem(const char *command, ErrorHandler *errh)
 {
@@ -89,6 +125,43 @@ mysystem(const char *command, ErrorHandler *errh)
     }
 }
 
+int
+temporary_file(String &filename, ErrorHandler *errh)
+{
+    if (no_create)
+	return 0;		// random number suffices
+    
+#ifdef HAVE_MKSTEMP
+    const char *tmpdir = getenv("TMPDIR");
+    if (tmpdir)
+	filename = String(tmpdir) + "/otftotfm.XXXXXX";
+    else {
+# ifdef P_tmpdir
+	filename = P_tmpdir "/otftotfm.XXXXXX";
+# else
+	filename = "/tmp/otftotfm.XXXXXX";
+# endif
+    }
+    int fd = mkstemp(filename.mutable_c_str());
+    if (fd < 0)
+	errh->error("temporary file '%s': %s", filename.c_str(), strerror(errno));
+    return fd;
+#else  // !HAVE_MKSTEMP
+    for (int tries = 0; tries < 5; tries++) {
+	if (!(filename = tmpnam(0)))
+	    return errh->error("cannot create temporary file");
+# ifdef O_EXCL
+	int fd = ::open(filename.c_str(), O_RDWR | O_CREAT | O_EXCL | O_TRUNC, 0600);
+# else
+	int fd = ::open(filename.c_str(), O_RDWR | O_CREAT | O_TRUNC, 0600);
+# endif
+	if (fd >= 0)
+	    return fd;
+    }
+    return errh->error("temporary file '%s': %s", filename.c_str(), strerror(errno));
+#endif
+}
+
 bool
 parse_unicode_number(const char* begin, const char* end, int require_prefix, uint32_t& result)
 {
