@@ -39,6 +39,10 @@
 # include <fcntl.h>
 # include <io.h>
 #endif
+#ifdef WIN32
+/* According to Fabrice Popineau MSVCC doesn't handle std::min correctly. */
+# define std /* */
+#endif
 
 using namespace Efont;
 
@@ -85,8 +89,8 @@ glyph in the font, including its glyph name and encoding.\n\
 \n\
 Usage: %s [OPTION]... [FONT]\n\
 \n\
-FONT is either the name of a PFA or PFB font file. If omitted, t1testpage will\n\
-read a font file from the standard input.\n\
+FONT is the name of a PFA or PFB font file. If omitted, t1testpage will read\n\
+a font file from the standard input.\n\
 \n\
 Options:\n\
   -g, --glyph=GLYPH            Limit output to one or more GLYPHs.\n\
@@ -95,7 +99,7 @@ Options:\n\
   -h, --help                   Print this message and exit.\n\
       --version                Print version number and exit.\n\
 \n\
-Report bugs to <kohler@icir.org>.\n", program_name);
+Report bugs to <kohler@cs.ucla.edu>.\n", program_name);
 }
 
 
@@ -468,8 +472,23 @@ extern "C" {
 static int CDECL
 glyphcompare(const void *lv, const void *rv)
 {
-    const PermString *ln = (const PermString *)lv;
-    const PermString *rn = (const PermString *)rv;
+    const PermString* ln = (const PermString*)lv;
+    const PermString* rn = (const PermString*)rv;
+
+    // try first without the '.'s
+    const char* ldot = strchr(ln->c_str(), '.');
+    const char* rdot = strchr(rn->c_str(), '.');
+    if (ldot == ln->begin())
+	ldot = 0;
+    if (rdot == rn->begin())
+	rdot = 0;
+    if (ldot || rdot) {
+	PermString l(ln->begin(), ldot ? ldot : ln->end());
+	PermString r(rn->begin(), rdot ? rdot : rn->end());
+	int diff = glyphcompare(&l, &r);
+	if (diff != 0)
+	    return diff;
+    }
 
     int lorder = glyph_order[*ln];
     int rorder = glyph_order[*rn];
